'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _ = require('lodash');
var jQuery = require('jquery');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var ___default = /*#__PURE__*/_interopDefaultLegacy(_);
var jQuery__default = /*#__PURE__*/_interopDefaultLegacy(jQuery);

function getAttrs(xml) {
  let attributes = {};
  [...xml.attributes].forEach(item => {
    attributes[item.name] = item.value.split(' ').filter(item => ___default['default'].trim(item)).map(item => ___default['default'].trim(item));
  });
  return attributes;
}

class PropsWatcher {
  constructor() {
    /**
     * {
     *  0: {
     *    myclick: [fn1, fn2, fn3]
     *  }
     * }
     */
    this.listenMap = {};
  }
  /**
   * 
   * @param {*} id 组件id
   * @param {*} key 事件名称
   * @param {*} fn 回调函数
   */
  on(compId, key, fn) {
    let compListener = this.listenMap[compId];
    // 事件名称必须是字符串，事件监听回调必须是函数
    if (___default['default'].isString(key) && ___default['default'].isFunction(fn)) {
      if (!___default['default'].isObject(compListener)) {
        this.listenMap[compId] = {};
      }
      if (!___default['default'].isArray(this.listenMap[compId][key])) {
        this.listenMap[compId][key] = [];
      }
      this.listenMap[compId][key].push(fn);
    }
  }

  /**
   * 事件触发
   * @param {*} compId 组件id
   * @param {*} key 事件名称
   * @param {*} data 返回值
   */
  emit(compId, key, data) {
    let compListener = this.listenMap[compId];
    if (compId == 'all') {
      Object.keys(this.listenMap).forEach((cid) => {
        this.emit(cid, key, data);
      });
      return;
    } 
    if (___default['default'].isObject(compListener) && ___default['default'].isArray(compListener[key]) && compListener[key].length > 0) {
      compListener[key].forEach(fn => {
        fn(data);
      });
    }
  }
}

const COMPONENT_ID = 'component-id';

const COMPONENT_ROUTER = 'component-router';

const createdID = () => {
  let id = 0;
  return () => {
    return id++;
  }
};
let JqComponent_ID = createdID();

function create$$($dom) {
  return function find(selector) {
    let childen = $dom.find(selector);
    $dom.parent().find(selector).each((index, $d) => {
      if ($dom.get(0) === $d) {
        childen.push($d);
      }
    });
    return childen;
  };
}

function setAttr($fragment, attrs) {
  Object.keys(attrs).forEach(key => {
    let values = attrs[key];
    values.forEach(v => {
      let lastedVal = $fragment.attr(key);
      let attrVals = lastedVal ? `${lastedVal} ${v}` : v;
      $fragment.attr(key, attrVals);
    });
  });
}

function mountedStyles($styles, tagName, jqcId) {
  jQuery__default['default']("head").append($styles);
  setComponentInfo2Dom($styles, ___default['default'].toLower(tagName), jqcId);
}

function proxy$$($$) {
  var handler = {
    get(target, key) {
      if (!target[key]) {
        return jQuery__default['default'][key];
      }
      return target[key];
    }
  };
  return new Proxy($$, handler);
}



const setComponentInfo2Dom = (dom, tagName, id) => {
  jQuery__default['default'](dom).attr(COMPONENT_ID, id);
};

// const eventHooks = function (id, dom, attrs) {
//   Object.keys(attrs).forEach(attr => {
//     if (_.startsWith(attr, 'on')) {
//       dom[attr] = () => {
//         let lisfn = attrs[attr][0];
//         var fnName = lisfn.replace(/\(.+?\)/g, '')
//         var arg = lisfn.match(/\((.+?)\)/g);
//         arg = arg[0].replace(/\(|\)/g, '').split(',').map(item => _.trim(item));
//         listenProps.emit(id, fnName, arg, dom);
//       }
//     }
//   })
// }

let allComponents = {};

let globalComponents = {};

const listenProps = new PropsWatcher();

const setGlobalComponent = function (name, jqc) {
  globalComponents[name] = jqc;
  return globalComponents;
};

const removeGlobalComponent = function (name) {
  delete globalComponents[name];
  return globalComponents;
};
class JqComponent {
  constructor(options = {}) {
    this.options = options;
    let { template, styles, components, ready } = this.options;
    this.template = template;
    this.$fragment = jQuery__default['default'](template);
    this.components = globalComponents;
    this.$styles = jQuery__default['default'](styles);
    this.ready = ready;
    this.id = JqComponent_ID();

    Object.keys(components || {}).forEach(key => {
      this.components[___default['default'].toUpper(key)] = components[key];
    });

    allComponents[this.id] = this;
  }

  render(root) {
    const tagName = root.tagName;
    if (tagName) {
      const options = this.components[tagName];
      const attrs = getAttrs(root);
   
      if (options) {
        const jqc = new JqComponent(options);
        setAttr(jqc.$fragment, attrs);

        root.parentNode.replaceChild(jqc.$fragment.get(0), root);
        setComponentInfo2Dom(jqc.$fragment, tagName, jqc.id);
        mountedStyles(jqc.$styles, tagName, jqc.id);
        jqc.render(jqc.$fragment.get(0));

        if (___default['default'].isFunction(jqc.ready)) {
          const $$ = create$$(jqc.$fragment);
          const watcher = (key, val) => listenProps.on(jqc.id, key, val);
          $$.jqc = jqc;
          $$.on = watcher;
          $$.el = jqc.$fragment;
          jqc.ready.apply(jqc.$fragment, [proxy$$($$)]);
        }
      }
    }
    root.childNodes.forEach(child => {
      this.render(child);
    });
  }
}

var store = {};

function sortRouters(routers) {
  routers.sort((a, b) => a.path < b.path ? -1 : a.path > b.path ? 1 : 0).reverse();  for (let router of routers) {
    if (Array.isArray(router.children)) {
      sortRouters(router.children);
    }
  }
  return routers;
}

const setRouters = (routers) => {
  store.routers = sortRouters(routers);
};

const getRouters = () => {
  return store.routers;
};

function mountedComponet($, routers, base = '') {
  $.el.html('');

  let hash = location.hash.replace('#', '') || '/';

  const hasParentRouter = $.el.parent().closest(`[${COMPONENT_ROUTER}]`).attr(COMPONENT_ROUTER);

  for (let router of routers) {
    let { path, component, children } = router;
    let fullPath = `${base}${path}`;
    if (___default['default'].startsWith(hash, fullPath)) {
      if (hasParentRouter) {
        // 嵌套路由，继续找子路由
        if (___default['default'].isArray(children)) {
          return mountedComponet($, children, fullPath);
        }
      }
      $.el.attr(COMPONENT_ROUTER, fullPath);
      $.el.html(component);
      listenProps.emit('all', 'router', { ...router, hash, path: fullPath });
      return;
    }
  }
}

var router = {
  template: `<div ${COMPONENT_ROUTER}></div>`,
  ready($) {
    const listener = (evt) => routerChange();

    $.on('destroy', () => window.removeEventListener('hashchange', listener));
    window.addEventListener('hashchange', listener);

    routerChange();

    function routerChange() {
      let routers = getRouters();
      mountedComponet($, routers);
    }  }
};

const useJquery = jQuery => {

  function findInnerComponents($dom) {
    return $dom.find(`[${COMPONENT_ID}]`);
  }
  
  function broadcastDestroyComponents($dom) {
    const $domComponents = findInnerComponents($dom);
    $domComponents.each((index, dom) => {
      const componentId = jQuery(dom).attr(COMPONENT_ID);
      listenProps.emit(componentId, 'destroy', { componentId, dom });
    });
  }

  const $html = jQuery.prototype.html;
  const $attr = jQuery.prototype.attr;
  const $data = jQuery.prototype.data;

  jQuery.component = function (tagName, conf) {
    return setGlobalComponent(___default['default'].toUpper(tagName), conf);
  };

  jQuery.unComponent = function(tagName) {
    return removeGlobalComponent(___default['default'].toUpper(tagName))
  };

  jQuery.router = function (routers, tagName = 'router-view') {
    setRouters(routers);
    jQuery.component(tagName, router);
  };

  jQuery.prototype.attr = function () {
    let args = [...arguments];
    let [key, val] = args;
    let componentId = $attr.call(this, COMPONENT_ID);
    if (componentId) {
      listenProps.emit(componentId, key, val);
    }
    return $attr.apply(this, args);
  };

  jQuery.prototype.data = function () {
    let args = [...arguments];
    let [key, val] = args;
    let componentId = $attr.call(this, COMPONENT_ID);
    if (componentId) {
      listenProps.emit(componentId, key, val);
    }
    return $data.apply(this, args)
  };

  jQuery.prototype.html = function (obj) {
    // 通知销毁组件
    broadcastDestroyComponents(this);
    if (___default['default'].isObject(obj)) {
      // 在根节点下动态生成一个占位组件标签
      const templateApp = 'template-' + Date.now() + '-' +  ___default['default'].random(0, 1000);
      $html.call(this, `<${templateApp}></${templateApp}>`);
      // 注册这个组件
      jQuery.component(templateApp, obj);
      // 则生成组件实例，再渲染
      let jqc = new JqComponent(obj);
      jqc.render(this.get(0));
      // 销毁动态组件
      jQuery.unComponent(templateApp);
      return this;
    } else {
      return $html.call(this, obj)
    }
  };
};

exports.useJquery = useJquery;
